@@ -9,7 +9,7 @@ use Moo;
 use Perinci::Object;
 use Perinci::ToUtil;
 
-our $VERSION = '0.43'; # VERSION
+our $VERSION = '0.44'; # VERSION
 
 with 'Perinci::To::Text::AddDocLinesRole';
 with 'SHARYANTO::Role::Doc::Section';
@@ -36,6 +36,8 @@ has custom_completer => (is => 'rw');
 has custom_arg_completer => (is => 'rw');
 has dash_to_underscore => (is => 'rw', default=>sub{1});
 has undo => (is=>'rw', default=>sub{0});
+has formats => (is => 'rw');
+
 has format => (is => 'rw', default=>sub{'text'});
 has _pa => (
     is => 'rw',
@@ -49,46 +51,57 @@ has _pa => (
 sub BUILD {
     my ($self, $args) = @_;
     #$self->{indent} = $args->{indent} // "    ";
+
+    if (!$self->{formats}) {
+        my $format_text = sub {
+            require Data::Format::Pretty;
+
+            my ($format, $res) = @_;
+            if (!defined($res->[2])) {
+                return $res->[0] == 200 ? "" :
+                    "ERROR $res->[0]: $res->[1]" .
+                        ($res->[1] =~ /\n\z/ ? "" : "\n");
+            }
+            my $r = $res->[0] == 200 ? $res->[2] : $res;
+            if ($format eq 'text') {
+                return Data::Format::Pretty::format_pretty(
+                    $r, {module=>'Console'});
+            }
+            if ($format eq 'pretty') {
+                return Data::Format::Pretty::format_pretty(
+                    $r, {module=>'Text'});
+            }
+            if ($format eq 'nopretty') {
+                return Data::Format::Pretty::format_pretty(
+                    $r, {module=>'SimpleText'});
+            }
+        };
+        $self->{formats} = {
+            yaml     => 'YAML',
+            json     => 'CompactJSON',
+            text     => $format_text,
+            pretty   => $format_text,
+            nopretty => $format_text,
+        };
+    };
 }
 
 sub format_result {
-    require Data::Format::Pretty;
-    *format_pretty = \&Data::Format::Pretty::format_pretty;
-
     my ($self) = @_;
-    my $format = $self->format;
 
-    if ($format eq 'yaml') {
-        $self->{_fres} = format_pretty($self->{_res}, {module=>'YAML'});
-        return;
-    }
-    if ($format eq 'json') {
-        $self->{_fres} = format_pretty($self->{_res}, {module=>'JSON'});
-        return;
-    }
-    if ($format =~ /^(text|pretty|nopretty)$/) {
-        if (!defined($self->{_res}[2])) {
-            $self->{_fres} = $self->{_res}[0] == 200 ? "" :
-                "ERROR $self->{_res}[0]: $self->{_res}[1]" .
-                    ($self->{_res}[1] =~ /\n\z/ ? "" : "\n");
-            return;
-        }
-        my $r = $self->{_res}[0] == 200 ? $self->{_res}[2] : $self->{_res};
-        if ($format eq 'text') {
-            $self->{_fres} = format_pretty($r, {module=>'Console'});
-            return;
-        }
-        if ($format eq 'pretty') {
-            $self->{_fres} = format_pretty($r, {module=>'Text'});
-            return;
-        }
-        if ($format eq 'nopretty') {
-            $self->{_fres} = format_pretty($r, {module=>'SimpleText'});
-            return;
-        }
-    }
+    my $format    = $self->format;
+    my $formats   = $self->formats;
+    my $formatter = $formats->{$format};
+    die "ERROR: Unknown output format '$format', please choose one of: ".
+        join(", ", sort keys(%$formats))."\n" unless $formatter;
 
-    die "BUG: Unknown output format `$format`";
+    if (ref($formatter) eq 'CODE') {
+        $self->{_fres} = $formatter->($format, $self->{_res});
+    } else {
+        require Data::Format::Pretty;
+        $self->{_fres} = Data::Format::Pretty::format_pretty(
+            $self->{_res}, {module=>$formatter});
+    }
 }
 
 sub get_subcommand {
@@ -260,8 +273,22 @@ sub run_completion {
         }
         $log->tracef("cleaned words=%s, cword=%d", $words, $cword);
 
+        # convert %getopts' ('help|h|?' => ..., ...) to ['--help', '-h', '-?',
+        # ...]. XXX this should be moved to another module to remove
+        # duplication, as Perinci::Sub::GetArgs::Argv also does something
+        # similar.
+        my $common_opts = [];
+        for my $k (keys %{$self->{_getopts_common}}) {
+            $k =~ s/^--?//;
+            $k =~ s/^([\w-]+(?:\|[\w-]+)*)(?:\W.*)?/$1/;
+            for (split /\|/, $k) {
+                push @$common_opts, (length == 1 ? "-$_" : "--$_");
+            }
+        }
+
         $res = Perinci::BashComplete::bash_complete_riap_func_arg(
             url=>$sc->{url}, words=>$words, cword=>$cword,
+            common_opts => $common_opts,
             custom_completer=>$self->custom_completer,
             custom_arg_completer => $self->custom_arg_completer
         );
@@ -365,6 +392,7 @@ Common options:
     --pretty, -p    Format result as pretty formatted text
     --nopretty, -P  Format result as simple formatted text
     --text         (Default) Select --pretty, or --nopretty when run piped
+    --format=FMT    Choose output format
 _
     $self->add_doc_lines($self->loc($text), "");
 }
@@ -533,11 +561,12 @@ sub _parse_common_opts {
         "json|j"     => sub { $self->format('json')     },
         "pretty|p"   => sub { $self->format('pretty')   },
         "nopretty|P" => sub { $self->format('nopretty') },
+        "format=s"   => sub { $self->format($_[1])      },
     );
 
     # convenience for Log::Any::App-using apps
     if ($self->log_any_app) {
-        for (qw/quiet verbose debug trace log_level/) {
+        for (qw/quiet verbose debug trace log-level/) {
             $getopts{$_} = sub {};
         }
     }
@@ -724,7 +753,7 @@ Perinci::CmdLine - Rinci/Riap-based command-line application framework
 
 =head1 VERSION
 
-version 0.43
+version 0.44
 
 =head1 SYNOPSIS
 
@@ -820,6 +849,13 @@ features. --undo-dir is used to set location of undo data (default C<~/.undo>;
 undo directory will be created if not exists; each subroutine will have its own
 subdir here).
 
+=head2 formats => HASH
+
+A hash containing mapping of format names and Data::Format::Pretty:: formatter
+module. By default, these formats are defined: C<yaml> => 'YAML', C<json> =>
+'CompactJSON', C<text> => 'Console', C<pretty> => 'Text', C<nopretty> =>
+'SimpleText'.
+
 =head1 METHODS
 
 =head2 new(%opts) => OBJ
@@ -911,6 +947,17 @@ versus:
 Though YAML requires spaces in some places where JSON does not. A flag to parse
 as JSON can be added upon request.
 
+=head2 How to add support for new output format (e.g. XML, HTML)?
+
+First make sure that Data::Format::Pretty::<FORMAT> module is available for your
+format. Look on CPAN. If it's not, i't also not hard to create one.
+
+Then, in your subclass' BUILD (or elsewhere), add a key to C<formats> hash
+attribute:
+
+ # this means format named 'xml' will be handled by Data::Format::Pretty::XML
+ $self->formats->{xml} = 'XML';
+
 =head1 SEE ALSO
 
 L<Perinci>, L<Rinci>, L<Riap>.
